@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 """
-Copyright (c) 2015 - 2020 Jolle Jolles <j.w.jolles@gmail.com>
+Copyright (c) 2015 - 2022 Jolle Jolles <j.w.jolles@gmail.com>
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ from __future__ import print_function
 from builtins import input
 
 import os
+import io
 import cv2
 import sys
 import yaml
@@ -41,6 +42,24 @@ from .camconfig import Camconfig
 from .schedule import Schedule
 from .__version__ import __version__
 
+class VidOutput(object):
+
+    """
+    Video output object for continuous monitoring of file size while recording
+    """
+
+    def __init__(self, filename):
+        self.vid = io.open(filename, 'wb')
+        self.size = 0
+
+    def write(self, s):
+        self.vid.write(s)
+        self.size += len(s)
+
+    def flush(self):
+        self.vid.flush()
+
+
 class PiRecorder:
 
     """
@@ -86,7 +105,7 @@ class PiRecorder:
             self.log.start()
             print("")
 
-        lineprint("pirecorder "+__version__+" started!")
+        lineprint("pirecorder "+__version__+" started!", date=True)
         lineprint("="*47, False)
 
         self.brightfile = self.setupdir+"/cusbright.yml"
@@ -106,7 +125,8 @@ class PiRecorder:
                           shutterspeed=8000,imgdims=(2592,1944),maxres=None,
                           viddims=(1640,1232),imgfps=1,vidfps=24,imgwait=5.0,
                           imgnr=12,imgtime=60,imgquality=50,vidduration=10,
-                          viddelay=10,vidquality=11,automode=True,internal="")
+                          viddelay=10,vidquality=11,automode=True,internal="",
+                          maxviddur=3600,maxvidsize=0)
             lineprint("Config settings stored..")
 
         else:
@@ -193,6 +213,8 @@ class PiRecorder:
         self.rawCapture = picamera.array.PiRGBArray(self.cam,
                           size=self.cam.resolution)
 
+        self.maxvidsize = self.config.vid.maxvidsize if self.config.vid.maxvidsize>0 else 999
+
 
     def _imgparams(self, mintime = 0.45):
 
@@ -295,9 +317,10 @@ class PiRecorder:
             and dynamically for each recording.
         maxres : str or tuple, default = "v2"
             The maximum potential resolution of the camera used. Either provide
-            a tuple of the max resolution, or use "v1.5", "v2" (default), or
-            "hq" to get the maximum resolution associated with the official
-            cameras directly.
+            a tuple of the max resolution, or use "v1.3", "v1.5", "v2" (default)
+            or "hq" to get the maximum resolution associated with the official
+            cameras directly. Note that "v2" is the default so when an older
+            camera model is connected this should be set here.
         rotation : int, default = 0
             Custom rotation specific to the Raspberry Pi, should be either 0 or
             180.
@@ -389,6 +412,14 @@ class PiRecorder:
             Specifies the quality that the h264 encoder should attempt to
             maintain. Use values between 10 and 40, where 10 is extremely high
             quality, and 40 is extremely low.
+        maxviddur : int, default = 3600
+            The maximum duration in seconds for single videos, beyond which
+            videos will be automatically split. A value of 0 indicates there is
+            no maximum file duration.
+        maxvidsize : int, default = 0
+            The maximum file size in Megabytes for single videos, beyond which
+            videos will be automatically split. A value of 0 indicates there is
+            no maximum file size.
         """
 
         if "recdir" in kwargs:
@@ -401,11 +432,15 @@ class PiRecorder:
             self.config.rec.rectype = kwargs["rectype"]
         if "maxres" in kwargs:
             self.config.rec.maxres = kwargs["maxres"]
-            if isinstance(self.config.rec.maxres, tuple):
+            if self.config.rec.maxres[0] != "(":
+                self.config.rec.imgdims = literal_eval(self.config.rec.maxres)
+            elif isinstance(self.config.rec.maxres, tuple):
                 self.config.img.imgdims = self.config.rec.maxres
-            elif self.config.rec.maxres == "v2":
+            if self.config.rec.maxres in ("v1.5","v1.3"):
+                self.config.img.imgdims = (2592,1944)
+            if self.config.rec.maxres == "v2":
                 self.config.img.imgdims = (3264,2464)
-            elif self.config.rec.maxres == "hq":
+            if self.config.rec.maxres == "hq":
                 self.config.img.imgdims = (4056,3040)
         if "rotation" in kwargs:
             self.config.cus.rotation = kwargs["rotation"]
@@ -455,14 +490,18 @@ class PiRecorder:
             self.config.vid.vidduration = kwargs["vidduration"]
         if "viddelay" in kwargs:
             self.config.vid.viddelay = kwargs["viddelay"]
-        if "vidquality" in kwargs:
-            self.config.vid.vidquality = kwargs["vidquality"]
+        if "maxviddur" in kwargs:
+            self.config.vid.maxviddur = kwargs["maxviddur"]
+        if "maxvidsize" in kwargs:
+            self.config.vid.maxvidsize = kwargs["maxvidsize"]
 
         brightchange = False
         if os.path.exists(self.brightfile):
             with open(self.brightfile) as f:
                 brighttune = yaml.load(f, Loader=yaml.FullLoader)
                 if brighttune != self.config.cus.brighttune:
+                    if "internal" not in kwargs:
+                        lineprint("cusbright.yml file found and loaded..")
                     self.config.cus.brighttune = brighttune
                     brightchange = True
 
@@ -493,6 +532,7 @@ class PiRecorder:
         else:
             lineprint("No roi selected..")
 
+
     def camconfig(self, fps=None, vidsize=0.4):
 
         lineprint("Opening stream for interactive configuration..")
@@ -586,11 +626,13 @@ class PiRecorder:
 
         elif self.config.rec.rectype == "imgseq":
 
-            timepoint = datetime.now()
+            starttime = datetime.now()
+            timepoint = starttime
             for i, img in enumerate(self.cam.capture_continuous(self.filename,
                                     format="jpeg", resize = self.resize,
                                     quality = self.config.img.imgquality)):
-                if i < self.config.img.imgnr-1:
+                tottimepassed = (datetime.now() - starttime).total_seconds()
+                if i < self.config.img.imgnr-1 and tottimepassed < self.config.img.imgtime:
                     timepassed = (datetime.now() - timepoint).total_seconds()
                     delay = max(0, self.config.img.imgwait - timepassed)
                     lineprint("Captured "+img+", sleeping "+str(round(delay,2))+"s..")
@@ -610,14 +652,33 @@ class PiRecorder:
 
             for session in ["_S%02d" % i for i in range(1,999)]:
                 session = "" if self.config.rec.rectype == "vid" else session
-                filename = self.filename+strftime("%H%M%S")+session+self.filetype
-                self.cam.start_recording(filename, resize = self.resize,
-                                         quality = self.config.vid.vidquality,
-                                         level = "4.2")
-                lineprint("Start recording "+filename)
-                self.cam.wait_recording(self.config.vid.vidduration+self.config.vid.viddelay)
-                self.cam.stop_recording()
-                lineprint("Finished recording "+filename)
+                filename = self.filename+strftime("%H%M%S")+session
+                timeremaining = self.config.vid.vidduration+self.config.vid.viddelay
+                counter = 0
+                while timeremaining > 0:
+                    counter += 1
+                    waittime = timeremaining
+                    if self.config.vid.maxviddur > 0:
+                        waittime = min(timeremaining, self.config.vid.maxviddur)
+                    if waittime == timeremaining and self.config.vid.maxvidsize == 0:
+                        nr = ""
+                    else:
+                        nr = "_v"+str(counter).zfill(2)
+                    finalname = filename+nr+self.filetype
+                    video = VidOutput(finalname)
+                    self.cam.start_recording(video, resize = self.resize,
+                                             quality = self.config.vid.vidquality,
+                                             level = "4.2",
+                                             format = self.filetype[1:])
+                    lineprint("Start recording "+filename)
+                    rectime = 0
+                    while video.size < self.maxvidsize*1000000 and rectime < waittime:
+                        rectime += 0.1
+                        self.cam.wait_recording(0.1)
+                    timeremaining -= rectime
+                    self.cam.stop_recording()
+                    vidinfo = " ("+str(round(rectime))+"s; "+str(round(video.size/1000000,2))+"MB)"
+                    lineprint("Finished recording "+finalname+vidinfo)
                 if self.config.rec.rectype == "vid":
                     break
                 else:
